@@ -4,7 +4,7 @@ use v5.14;
 use warnings;
 use utf8;
 
-our $VERSION = "2.02";
+our $VERSION = "2.03";
 
 use Data::Dumper;
 use Carp;
@@ -29,8 +29,9 @@ sub ansi_fold {
 ######################################################################
 
 my $alphanum_re = qr{ [_\d\p{Latin}] }x;
-my $reset_re    = qr{ \e \[ [0;]* m (?: \e \[ [0;]* [mK])* }x;
-my $color_re    = qr{ \e \[ [\d;]* [mK] }x;
+my $reset_re    = qr{ \e \[ [0;]* m }x;
+my $color_re    = qr{ \e \[ [\d;]* m }x;
+my $erase_re    = qr{ \e \[ [\d;]* K }x;
 my $control_re  = qr{
     # see ECMA-48 8.3.89 OSC - OPERATING SYSTEM COMMAND
     \e \]			# osc
@@ -142,12 +143,15 @@ sub configure {
 }
 
 my @color_stack;
+my @bg_stack;
 my @reset;
 sub put_reset { @reset = shift };
 sub pop_reset {
     @reset ? do { @color_stack = (); pop @reset } : '';
 }
 
+use List::Util qw(pairgrep);
+
 sub fold {
     my $obj = ref $_[0] ? $_[0] : do {
 	state $private = configure();
@@ -155,20 +159,9 @@ sub fold {
     shift;
 
     local $_ = shift // '';
-    my %opt = @_;
-
-    my $width     = $opt{width}     // $obj->{width};
-    my $boundary  = $opt{boundary}  // $obj->{boundary};
-    my $padding   = $opt{padding}   // $obj->{padding};
-    my $padchar   = $opt{padchar}   // $obj->{padchar};
-    my $ambiguous = $opt{ambiguous} // $obj->{ambiguous};
-    my $margin    = $opt{margin}    // $obj->{margin};
-    my $linebreak = $opt{linebreak} // $obj->{linebreak};
-    my $runin     = $opt{runin}     // $obj->{runin};
-    my $runout    = $opt{runout}    // $obj->{runout};
-    my $expand    = $opt{expand}    // $obj->{expand};
-    my $tabstop   = $opt{tabstop}   // $obj->{tabstop};
+    my %opt = ( %$obj, pairgrep { defined $b } @_ );
 
+    my $width = $opt{width} // die;
     if ($width < 0) {
 	$width = ~0 >> 1; # INT_MAX
     }
@@ -177,18 +170,18 @@ sub fold {
 	croak "invalid width";
     }
 
-    if ($width <= $margin) {
+    if ($width <= $opt{margin}) {
 	croak "invalid margin";
     }
-    $width -= $margin;
+    $width -= $opt{margin};
 
-    $Text::VisualWidth::PP::EastAsian = $ambiguous eq 'wide';
+    $Text::VisualWidth::PP::EastAsian = $opt{ambiguous} eq 'wide';
 
     my $folded = '';
     my $eol = '';
     my $room = $width;
-    @color_stack = @reset = ();
-    my $yield_re = $expand ? qr/[^\e\n\f\r\t]/ : qr/[^\e\n\f\r]/;
+    @bg_stack = @color_stack = @reset = ();
+    my $yield_re = $opt{expand} ? qr/[^\e\n\f\r\t]/ : qr/[^\e\n\f\r]/;
 
   FOLD:
     while (length) {
@@ -198,7 +191,7 @@ sub fold {
 	    $eol = $1;
 	    last;
 	}
-	# formfeed
+	# formfeed / carriage return
 	if (s/\A([\f\r]+)//) {
 	    last if length == 0;
 	    $folded .= $1;
@@ -210,9 +203,16 @@ sub fold {
 	    $folded .= $1;
 	    next;
 	}
+	# erase line (assume 0)
+	if (s/\A($erase_re)//) {
+	    $folded .= $1;
+	    @bg_stack = @color_stack;
+	    next;
+	}
 	# reset
-	if (s/\A($reset_re)//) {
+	if (s/\A($reset_re+($erase_re*))//) {
 	    put_reset($1);
+	    @bg_stack = () if $2;
 	    next;
 	}
 
@@ -231,8 +231,9 @@ sub fold {
 	}
 
 	# tab
-	if ($expand and s/\A(\t+)//) {
-	    my $space = $tabstop * length($1) - ($width - $room) % $tabstop;
+	if ($opt{expand} and s/\A(\t+)//) {
+	    my $space =
+		$opt{tabstop} * length($1) - ($width - $room) % $opt{tabstop};
 	    $_ = ' ' x $space . $_;
 	    next;
 	}
@@ -276,7 +277,7 @@ sub fold {
 	}
     }
 
-    if ($boundary eq 'word'
+    if ($opt{boundary} eq 'word'
 	and my($tail) = /^(${alphanum_re}+)/o
 	and $folded =~ m{
 		^
@@ -300,11 +301,11 @@ sub fold {
     ## RUN-OUT
     ##
     if ($_ ne ''
-	and $linebreak & LINEBREAK_RUNOUT and $runout > 0
+	and $opt{linebreak} & LINEBREAK_RUNOUT and $opt{runout} > 0
 	and $folded =~ m{ (?<color>  (?! ${reset_re}) ${color_re}*+ )
 			  (?<runout> $prohibition_re{end}+ ) \z }xp
 	and ${^PREMATCH} ne ''
-	and (my $w = vwidth $+{runout}) <= $runout) {
+	and (my $w = vwidth $+{runout}) <= $opt{runout}) {
 	$folded = ${^PREMATCH};
 	$_ = join '', ${^MATCH}, @reset, $_;
 	pop_reset() if $+{color};
@@ -313,14 +314,14 @@ sub fold {
 
     $folded .= pop_reset() if @reset;
 
-    $room += $margin;
+    $room += $opt{margin};
 
     ##
     ## RUN-IN
     ##
-    if ($linebreak & LINEBREAK_RUNIN and $runin > 0) {
+    if ($opt{linebreak} & LINEBREAK_RUNIN and $opt{runin} > 0) {
 	my @runin;
-	my $m = $runin;
+	my $m = $opt{runin};
 	while ($m > 0 and
 	       m{\A (?<color> ${color_re}*+)
 	            (?<runin> $prohibition_re{head})
@@ -342,8 +343,12 @@ sub fold {
 	$_ = join '', @color_stack, $_ if $_ ne '';
     }
 
-    if ($padding and $room > 0) {
-	$folded .= $padchar x $room;
+    if ($opt{padding} and $room > 0) {
+	my $padding = $opt{padchar} x $room;
+	if (@bg_stack) {
+	    $padding = join '', @bg_stack, $padding, SGR_RESET;
+	}
+	$folded .= $padding;
     }
 
     ($folded . $eol, $_, $width - $room);
@@ -433,7 +438,7 @@ Text::ANSI::Fold - Text folding library supporting ANSI terminal sequence and As
 
 =head1 VERSION
 
-Version 2.02
+Version 2.03
 
 =head1 SYNOPSIS
 
@@ -604,6 +609,10 @@ shorter than 80.
 
     ansi_fold($text, 80, padding => 1);
 
+If an ANSI B<Erase Line> sequence is found in the string, color status
+at the position is remembered, and padding string is produced in that
+color.
+
 =item B<padchar> => I<char>
 
 B<padchar> option specifies character used to fill up the remainder of
